from datetime import datetime
from typing import Dict, Self, Tuple

from enums import grade, position


class Player:
    def __init__(
        self,
        player_id: str = None,
        first_name: str = None,
        last_name: str = None,
        position: position.Position = None,
        number: int = None,
        grade: grade.Grade = None,
        school: str = None,
        imgurl: str = None,
        created: datetime = None,
        modified: datetime = None,
    ) -> Self:
        self.player_id = player_id
        self.first_name = first_name
        self.last_name = last_name
        self.position = position
        self.number = number
        self.grade = grade
        self.school = school
        self.imgurl = imgurl
        self.created = created
        self.modified = modified

    @classmethod
    def from_tuple(cls, player_tuple: Tuple) -> None:
        """Create player from a tuple

        Args:
            player_tuple (Tuple): Tuple containing ordered player data
        """
        return cls(**{k: v for k, v in zip(cls().to_dict().keys(), player_tuple)})

    def __str__(self) -> str:
        return (
            f"#{self.number} {self.first_name} {self.last_name} [{self.position}] "
            + f"| {self.grade} at {self.school}"
        )

    def to_dict(self) -> Dict:
        return {
            "player_id": f"{self.player_id}",
            "first_name": f"{self.first_name}",
            "last_name": f"{self.last_name}",
            "position": self.position,
            "number": self.number,
            "grade": self.grade,
            "school": f"{self.school}",
            "imgurl": f"{self.imgurl}",
            "created": f"{self.created}",
            "modified": f"{self.modified}",
        }



from datetime import datetime
from typing import Optional

from pydantic import BaseModel


class Coach(BaseModel):
    coach_id: Optional[str] = None
    first_name: Optional[str] = None
    last_name: Optional[str] = None
    role: Optional[str] = None
    since: Optional[int] = None
    email: Optional[str] = None
    phone_number: Optional[str] = None
    imgurl: Optional[str] = None
    created: Optional[datetime] = None
    modified: Optional[datetime] = None

    @property
    def name(self) -> str:
        return f"{self.first_name} {self.last_name}"



from typing import List

from pydantic import BaseModel

from .statistics import Statistics


class PlayerGameStatistics(BaseModel):
    player_id: str = None
    statistics: Statistics = None


class CompositeGameStatistics(BaseModel):
    game_id: str = None
    title: str = None
    statistics: List[PlayerGameStatistics] = []



from typing import Dict, List, Self


class TeamFilters:
    def __init__(
        self,
        team_ids: List[str] = None,
        names: List[str] = None,
        limit: int = 40,
        offset: int = 0,
        order: str = "ASC",
        order_by: str = "name",
    ) -> Self:
        self.team_ids = team_ids
        self.names = names
        self.limit = limit
        self.offset = offset
        self.order = order
        self.order_by = order_by

    def _should_add_contents_of_list(self, data: List) -> bool:
        return data is not None and len(data) > 0

    def to_dict(self) -> Dict:
        filters = {}

        if self._should_add_contents_of_list(self.team_ids):
            filters["filter.teamId"] = str.join(",", self.team_ids)
        if self._should_add_contents_of_list(self.names):
            filters["filter.name"] = str.join(",", self.names)

        filters["limit"] = self.limit
        filters["offset"] = self.offset
        filters["order"] = self.order
        filters["orderBy"] = self.order_by

        return filters



from datetime import datetime
from typing import Dict, Tuple

from enums import position


class GameResult:
    def __init__(
        self,
        game_id: str = None,
        home_team_id: str = None,
        away_team_id: str = None,
        title: str = None,
        date: datetime = None,
        score: str = None,
        location: str = None,
        team_id: str = None,
        team_name: str = None,
        team_image_url: str = None,
        player_id: str = None,
        first_name: str = None,
        last_name: str = None,
        position: position.Position = None,
        number: int = None,
        player_image_url: str = None,
        statistics: str = None,
    ):
        self.game_id = game_id
        self.home_team_id = home_team_id
        self.away_team_id = away_team_id
        self.title = title
        self.date = date
        self.score = score
        self.location = location
        self.team_id = team_id
        self.team_name = team_name
        self.team_image_url = team_image_url
        self.player_id = player_id
        self.first_name = first_name
        self.last_name = last_name
        self.position = position
        self.number = number
        self.player_image_url = player_image_url
        self.statistics = statistics

    @classmethod
    def from_tuple(cls, game_result_tuple: Tuple) -> None:
        return cls(**{k: v for k, v in zip(cls().to_dict().keys(), game_result_tuple)})

    def to_dict(self) -> Dict:
        return {
            "game_id": f"{self.game_id}",
            "home_team_id": f"{self.home_team_id}",
            "away_team_id": f"{self.away_team_id}",
            "title": f"{self.title}",
            "date": f"{self.date}",
            "score": f"{self.score}",
            "location": f"{self.location}",
            "team_id": f"{self.team_id}",
            "team_name": f"{self.team_name}",
            "team_image_url": f"{self.team_image_url}",
            "player_id": f"{self.player_id}",
            "first_name": f"{self.first_name}",
            "last_name": f"{self.last_name}",
            "position": self.position,
            "number": f"{self.number}",
            "player_image_url": f"{self.player_image_url}",
            "statistics": f"{self.statistics}",
        }



from typing import Iterable, Self


class PlayerDataServiceResponse:
    _status: int
    _data: Iterable

    def __init__(self, status: int, data: dict) -> Self:
        self._status = status
        self._data = data

    @property
    def status(self) -> int:
        return self._status

    @status.setter
    def status(self, new_status: int) -> None:
        self._status = new_status

    @property
    def data(self) -> Iterable:
        return self._data

    @data.setter
    def data(self, new_data: Iterable) -> None:
        self._data = new_data

    @data.setter
    def data(self, new_data: str) -> None:
        self._data = self.data_from_string(new_data)



from typing import Dict, List, Self

from .composite_game_statistics import CompositeGameStatistics
from .composite_season_statistics import CompositeSeasonStatistics


class CompositeStatistics:
    games: List[CompositeGameStatistics]
    season: List[CompositeSeasonStatistics]

    def __init__(
        self,
        game_stats: List[CompositeGameStatistics],
        season_stats: List[CompositeSeasonStatistics],
    ) -> Self:
        self.games = game_stats
        self.season = season_stats

    def to_dict(self) -> Dict:
        return {
            "games": self.games.to_dict(),
            "season": self.season.to_dict(),
        }



from datetime import datetime
from typing import Optional

from pydantic import BaseModel


class GameRequestFilters(BaseModel):
    game_id: Optional[str] = None
    date: Optional[datetime] = None
    limit: Optional[int] = None
    offset: Optional[int] = None
    order: Optional[str] = None
    order_by: Optional[str] = None



from typing import Dict, List, Tuple


class CompositeTeamPlayer:
    team_id: str
    team_name: str
    team_location: str
    year: int
    team_img_url: str
    player_id: str
    first_name: str
    last_name: str
    number: int
    position: str
    grade: str
    school: str
    img_url: str

    def __init__(
        self,
        team_id: str = None,
        team_name: str = None,
        team_img_url: str = None,
        team_location: str = None,
        year: int = None,
        player_id: str = None,
        first_name: str = None,
        last_name: str = None,
        number: int = None,
        position: str = None,
        grade: str = None,
        school: str = None,
        img_url: str = None,
    ):
        self.team_id = team_id
        self.team_name = team_name
        self.team_location = team_location
        self.team_img_url = team_img_url
        self.year = year
        self.player_id = player_id
        self.first_name = first_name
        self.last_name = last_name
        self.number = number
        self.position = position
        self.grade = grade
        self.school = school
        self.img_url = img_url

    @classmethod
    def from_tuple(cls, composite_team_player_tuple: Tuple) -> None:
        return cls(**{k: v for k, v in zip(cls().to_dict().keys(), composite_team_player_tuple)})

    def to_dict(self) -> Dict:
        return {
            "team_id": f"{self.team_id}",
            "team_name": f"{self.team_location}",
            "team_location": f"{self.team_location}",
            "team_img_url": f"{self.team_img_url}",
            "year": self.year,
            "player_id": f"{self.player_id}",
            "first_name": f"{self.first_name}",
            "last_name": f"{self.last_name}",
            "number": f"{self.number}",
            "position": f"{self.position}",
            "grade": f"{self.grade}",
            "school": f"{self.school}",
            "img_url": f"{self.img_url}",
        }


class CompositeTeamCoach:
    team_id: str
    team_name: str
    team_location: str
    team_img_url: str
    year: int
    coach_id: str
    first_name: str
    last_name: str
    role: str
    since: int
    email: str
    phone_number: str
    img_url: str

    def __init__(
        self,
        team_id: str = None,
        team_name: str = None,
        team_location: str = None,
        team_img_url: str = None,
        year: int = None,
        coach_id: str = None,
        first_name: str = None,
        last_name: str = None,
        role: str = None,
        since: int = None,
        email: str = None,
        phone_number: str = None,
        img_url: str = None,
    ):
        self.team_id = team_id
        self.team_name = team_name
        self.team_location = team_location
        self.team_img_url = team_img_url
        self.year = year
        self.coach_id = coach_id
        self.first_name = first_name
        self.last_name = last_name
        self.role = role
        self.since = since
        self.email = email
        self.phone_number = phone_number
        self.img_url = img_url

    @classmethod
    def from_tuple(cls, composite_team_coach_tuple: Tuple) -> None:
        return cls(**{k: v for k, v in zip(cls().to_dict().keys(), composite_team_coach_tuple)})

    def to_dict(self) -> Dict:
        return {
            "team_id": f"{self.team_id}",
            "team_name": f"{self.team_name}",
            "team_location": f"{self.team_location}",
            "team_img_url": f"{self.team_img_url}",
            "year": self.year,
            "coach_id": f"{self.coach_id}",
            "first_name": f"{self.first_name}",
            "last_name": f"{self.last_name}",
            "role": f"{self.role}",
            "since": f"{self.since}",
            "email": f"{self.email}",
            "phone_number": f"{self.phone_number}",
            "img_url": f"{self.img_url}",
        }


class CompositeTeam:
    players: List[CompositeTeamPlayer]
    coaches: List[CompositeTeamCoach]

    def __init__(
        self,
        players: List[CompositeTeamPlayer],
        coaches: List[CompositeTeamCoach],
    ):
        self.players = players
        self.coaches = coaches

    def to_dict(self) -> Dict:
        return {
            "players": [player.to_dict() for player in self.players],
            "coaches": [coach.to_dict() for coach in self.coaches],
        }



from datetime import datetime
from typing import Dict, Self, Tuple


class Game:
    def __init__(
        self,
        game_id: str = None,
        title: str = None,
        date: datetime = None,
        score: str = None,
        location: str = None,
        created: datetime = None,
        modified: datetime = None,
    ) -> Self:
        self.game_id = game_id
        self.title = title
        self.date = date
        self.score = score
        self.location = location
        self.created = created
        self.modified = modified

    @classmethod
    def from_tuple(cls, game_tuple: Tuple) -> None:
        """Create game from a tuple

        Args:
            player_tuple (Tuple): Tuple containing ordered game data
        """
        return cls(**{k: v for k, v in zip(cls().to_dict().keys(), game_tuple)})

    def __str__(self) -> str:
        return f"{self.date} | {self.title}: {self.score}"

    def to_dict(self) -> Dict:
        return {
            "game_id": f"{self.game_id}",
            "title": f"{self.title}",
            "date": f"{self.date}",
            "score": f"{self.score}",
            "location": f"{self.location}",
            "created": f"{self.created}",
            "modified": f"{self.modified}",
        }



from datetime import datetime
from typing import Optional

from pydantic import BaseModel


class Player(BaseModel):
    player_id: Optional[str] = None
    first_name: Optional[str] = None
    last_name: Optional[str] = None
    number: Optional[int] = None
    position: Optional[str] = None
    grade: Optional[str] = None
    school: Optional[str] = None
    imgurl: Optional[str] = None
    created: Optional[datetime] = None
    modified: Optional[datetime] = None

    @property
    def name(self) -> str:
        return f"{self.first_name} {self.last_name}"

    def to_dict(self, full_definition: bool = False) -> dict:
        player_dict = {
            "player_id": self.player_id,
            "first_name": self.first_name,
            "last_name": self.last_name,
            "position": self.position,
            "number": self.number,
            "grade": self.grade,
            "school": self.school,
            "img_url": self.imgurl,
        }

        if full_definition:
            player_dict.update({"created": self.created, "modified": self.modified})

        return player_dict



from typing import List


class GameValidationError(Exception):
    def __init__(self, message=None, invalid_fields: List = None):
        self.__message = message
        self.invalid_fields = invalid_fields
        super().__init__(self.__message)


class GameDoesNotExist(Exception):
    def __init__(self, message=None):
        self.__message = message
        super().__init__(self.__message)


class GameAlreadyExists(Exception):
    def __init__(self, message=None, existing_game_id: str = None) -> None:
        self.__message = message
        self.existing_game_id = existing_game_id
        super().__init__(self.__message)



from typing import List


class TeamValidationError(Exception):
    def __init__(self, message=None, invalid_fields: List = None):
        self.__message = message
        self.invalid_fields = invalid_fields
        super().__init__(self.__message)


class TeamDoesNotExist(Exception):
    def __init__(self, message=None, invalid_team_id: str = None):
        self.__message = message
        self.invalid_team_id = invalid_team_id
        super().__init__(self.__message)


class TeamAlreadyExists(Exception):
    def __init__(self, message=None, existing_team_id: str = None) -> None:
        self.__message = message
        self.existing_team_id = existing_team_id
        super().__init__(self.__message)



from typing import List, Optional

from pydantic import BaseModel

from .coach import Coach
from .player import Player


class Roster(BaseModel):
    year: Optional[int] = None
    players: Optional[List[Player]] = []
    coaches: Optional[List[Coach]] = []


class CompositeTeam(BaseModel):
    team_id: Optional[str] = None
    name: Optional[str] = None
    location: Optional[str] = None
    img_url: Optional[str] = "static/blank.jpg"
    roster: Optional[List[Roster]] = []



from datetime import datetime
from typing import Dict, Self, Tuple


class Team:
    def __init__(
        self,
        team_id: str = None,
        name: str = None,
        location: str = None,
        imgurl: str = None,
        created: datetime = None,
        modified: datetime = None,
    ) -> Self:
        self.team_id = team_id
        self.name = name
        self.location = location
        self.imgurl = imgurl
        self.created = created
        self.modified = modified

    @classmethod
    def from_tuple(cls, team_tuple: Tuple) -> None:
        """Create team from a tuple

        Args:
            player_tuple (Tuple): Tuple containing ordered team data
        """
        return cls(**{k: v for k, v in zip(cls().to_dict().keys(), team_tuple)})

    def __str__(self) -> str:
        return f"{self.name}"

    def to_dict(self) -> Dict:
        return {
            "team_id": f"{self.team_id}",
            "name": f"{self.name}",
            "location": f"{self.location}",
            "imgurl": f"{self.imgurl}",
            "created": f"{self.created}",
            "modified": f"{self.modified}",
        }



from typing import Dict, List, Self


class PlayersFilters:
    def __init__(
        self,
        player_ids: List[str] = None,
        first_names: List[str] = None,
        last_names: List[str] = None,
        numbers: List[str] = None,
        positions: List[str] = None,
        grades: List[str] = None,
        limit: int = 40,
        offset: int = 0,
        order: str = None,
        order_by: str = None,
    ) -> Self:
        self.player_ids = player_ids
        self.first_names = first_names
        self.last_names = last_names
        self.numbers = numbers
        self.positions = positions
        self.grades = grades
        self.limit = limit
        self.offset = offset
        self.order = order
        self.order_by = order_by

    def _should_add_contents_of_list(self, data: List) -> bool:
        return data is not None and len(data) > 0

    def to_dict(self) -> Dict:
        filters = {}

        if self._should_add_contents_of_list(self.player_ids):
            filters["filter.playerId"] = str.join(",", self.player_ids)
        if self._should_add_contents_of_list(self.first_names):
            filters["filter.firstName"] = str.join(",", self.first_names)
        if self._should_add_contents_of_list(self.last_names):
            filters["filter.lastName"] = str.join(",", self.last_names)
        if self._should_add_contents_of_list(self.positions):
            filters["filter.position"] = str.join(",", self.positions)
        if self._should_add_contents_of_list(self.numbers):
            filters["filter.number"] = str.join(",", self.numbers)
        if self._should_add_contents_of_list(self.grades):
            filters["filter.grade"] = str.join(",", self.grades)

        if self.limit is not None:
            filters["limit"] = self.limit
        if self.offset is not None:
            filters["offset"] = self.offset
        if self.order is not None:
            filters["order"] = self.order
        if self.order_by is not None:
            filters["orderBy"] = self.order_by

        return filters



from typing import Optional

from pydantic import BaseModel


class Statistics(BaseModel):
    """
    Create statistics DTO object. The statistics for each player
    is represented in the db as a string in this format:

    0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0
    """

    hsh: Optional[int] = 0
    msh: Optional[int] = 0
    lsh: Optional[int] = 0
    hg: Optional[int] = 0
    mg: Optional[int] = 0
    lg: Optional[int] = 0
    a: Optional[int] = 0
    gb: Optional[int] = 0
    t: Optional[int] = 0
    ct: Optional[int] = 0
    p: Optional[int] = 0
    k: Optional[int] = 0
    ms: Optional[int] = 0
    hga: Optional[int] = 0
    mga: Optional[int] = 0
    lga: Optional[int] = 0
    hgs: Optional[int] = 0
    mgs: Optional[int] = 0
    lgs: Optional[int] = 0
    fow: Optional[int] = 0
    fol: Optional[int] = 0

    @classmethod
    def from_string(cls, statistics_string: str):
        stats = dict(zip(cls.model_fields.keys(), map(int, statistics_string.split("|"))))
        return Statistics(**stats)

    def __str__(self) -> str:
        return (
            f"{self.hsh}|{self.msh}|{self.lsh}|{self.hg}|{self.mg}|{self.lg}|{self.a}|{self.gb}"
            + f"|{self.t}|{self.ct}|{self.p}|{self.k}|{self.ms}|{self.hga}|{self.mga}"
            + f"|{self.lga}|{self.hgs}|{self.mgs}|{self.lgs}|{self.fow}|{self.fol}"
        )



from typing import Dict, Optional, Self


class Pagination:
    _limit: Optional[int]
    _offset: Optional[int]

    def __init__(self, limit: int = None, offset: int = None) -> Self:
        self._limit = self._limit_value(limit)
        self._offset = self._offset_value(offset)

    def _limit_value(self, limit: int) -> int | None:
        if limit is None or limit < 0 or limit > 10000:
            return None
        return limit

    def _offset_value(self, offset: int) -> int | None:
        if offset is None or offset < 0 or offset > 10000:
            return None
        return offset

    @property
    def limit(self) -> int:
        return self._limit

    @limit.setter
    def limit(self, new_limit: int) -> None:
        self._limit = self._limit_value(new_limit)

    @property
    def offset(self) -> int:
        return self._offset

    @offset.setter
    def offset(self, new_offset) -> None:
        self._offset = self._offset_value(new_offset)

    def to_dict(self) -> Dict:
        data = {}

        if self._limit is not None:
            data["limit"] = self._limit
        if self._offset is not None:
            data["offset"] = self._offset

        return data



from typing import Dict, Self


class OrderingRules:
    def __init__(self, order: str = None, order_by: str = None) -> Self:
        self.order = order
        self.order_by = order_by

    def to_dict(self) -> Dict:
        data = {}

        if self.order is not None:
            data["order"] = self.order
        if self.order_by is not None:
            data["orderBy"] = self.order_by

        return data



from typing import Optional

from pydantic import BaseModel


class TeamRequestFilters(BaseModel):
    team_id: Optional[str] = None
    team_name: Optional[str] = None
    limit: Optional[int] = None
    offset: Optional[int] = None
    order: Optional[str] = None
    order_by: Optional[str] = None
    limit: Optional[int] = None
    offset: Optional[int] = None


class CompositeTeamRequestFilters(BaseModel):
    team_id: Optional[str] = None
    player_id: Optional[str] = None
    coach_id: Optional[str] = None
    year: Optional[int] = None
    team_name: Optional[str] = None
    player_name: Optional[str] = None
    order: Optional[str] = None
    order_by: Optional[str] = None
    limit: Optional[int] = None
    offset: Optional[int] = None
